@@ -2,9 +2,11 @@ mod avoid_empty_tables;
 mod build;
 mod dev;
 mod dry_run;
+mod gc_keep_used_patch;
 mod gc_patch;
 mod gc_profile;
 mod gc_replace;
+mod help;
 mod invalid_arg;
 mod invalid_dep;
 mod invalid_package;
